<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs" version="2.0">

    <xsl:output encoding="UTF-8" method="xml" indent="yes"/>
    
    <xsl:template match="/">
        <wkt-pi>
            <xsl:apply-templates select="//Folder"/>
        </wkt-pi>
    </xsl:template>
    
    <xsl:template match="Folder">
        <Folder>
            <xsl:copy-of select="description"/>
            <xsl:apply-templates select="Placemark"/>
        </Folder>
    </xsl:template>
    
    <xsl:template match="Placemark">
        <Placemark>
            <xsl:copy-of select="description"/>
            <xsl:apply-templates select="Polygon | MultiGeometry | Point"/>
        </Placemark>
    </xsl:template>
    
    <xsl:template match="MultiGeometry">
        <xsl:copy-of select="."/>
        <geo-wkt>
            <xsl:text>MULTIPOLYGON(</xsl:text>
            <xsl:for-each select="Polygon">
                <xsl:text>(</xsl:text>
                <xsl:apply-templates select="outerBoundaryIs/LinearRing"/>
                <xsl:for-each select="innerBoundaryIs/LinearRing">
                    <xsl:text>, </xsl:text>
                    <xsl:apply-templates select="."/>
                </xsl:for-each>
                <xsl:text>)</xsl:text>
                <xsl:choose>
                    <xsl:when test="position() = last()">
                        <xsl:text>)</xsl:text>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>, </xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </geo-wkt>    
    </xsl:template>
    
    <xsl:template match="Polygon">
        <xsl:copy-of select="."/>
        <geo-wkt>
            <xsl:text>POLYGON(</xsl:text>
            <xsl:apply-templates select="outerBoundaryIs/LinearRing"/>
            <xsl:for-each select="innerBoundaryIs/LinearRing">
                <xsl:text>, </xsl:text>
                <xsl:apply-templates select="."/>
            </xsl:for-each>
            <xsl:text>)</xsl:text>
        </geo-wkt>
    </xsl:template>
    
    <xsl:template match="LinearRing">
        <xsl:text>(</xsl:text>
        <xsl:for-each select="coordinate">
            <xsl:value-of select="@latitude"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="@longitude"/>
            <xsl:choose>
                <xsl:when test="position() = last()">
                    <xsl:text>)</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>, </xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each> 
    </xsl:template>

    <xsl:template match="Point">
        <xsl:copy-of select="."/>
        <geo-wkt>
            <xsl:text>POINT(</xsl:text>
            <xsl:value-of select="coordinate/@latitude"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="coordinate/@longitude"/>
            <xsl:text>)</xsl:text>
        </geo-wkt>
    </xsl:template>
</xsl:stylesheet>
