<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:saxon="http://saxon.sf.net/"
    xmlns:fn="http://www.w3.org/2005/xpath-functions"
    xmlns:kml="http://earth.google.com/kml/2.1"
    exclude-result-prefixes="saxon fn kml"
    version="2.0">
    <!-- Requires the use of saxxon:parse extension to interpret CDATA -->
    
    <xsl:output encoding="UTF-8" method="xml" indent="yes"/>
        
    <xsl:template match="/">
        <kml-pi>
            <xsl:apply-templates select="//kml:Folder"/>
        </kml-pi>    
    </xsl:template>
    
    <xsl:template match="kml:Folder">
        <Folder>
            <description>
                <xsl:value-of select="kml:description/text()"/>
            </description>
            <xsl:apply-templates select="kml:Placemark"/>
        </Folder>
    </xsl:template>
    
    <xsl:template match="kml:Placemark">
        <Placemark>
            <xsl:variable name="placemark-description-cdata">
                <xsl:value-of select="kml:description/text()"/>
            </xsl:variable>
            <!-- removes the <br> and close the <table> tag -->
            <xsl:variable name="placemark-description-cdata-well-formed">
                <xsl:value-of select="fn:concat(fn:replace($placemark-description-cdata,
                    '&lt;br&gt;&lt;br&gt;&lt;br&gt;',''), '&lt;/table&gt;')"/>
            </xsl:variable>
            <!-- read as well formed xml fragment -->
            <xsl:variable name="xml" select="saxon:parse($placemark-description-cdata-well-formed)"/>
            <description>
                <xsl:for-each select="$xml//tr">
                    <xsl:element name="meta-information">
                        <xsl:attribute name="attribute">
                            <xsl:value-of select="td[1]"/>
                        </xsl:attribute>
                        <xsl:attribute name="value">
                            <xsl:value-of select="td[2]"/>
                        </xsl:attribute>
                    </xsl:element>
                </xsl:for-each>
            </description>
            <xsl:apply-templates select="kml:Polygon | kml:MultiGeometry | kml:Point"/>
        </Placemark>
    </xsl:template>
    
    <xsl:template match="kml:Polygon">
        <Polygon>
            <xsl:apply-templates select="kml:outerBoundaryIs | kml:innerBoundaryIs"/>
        </Polygon>    
    </xsl:template>

    <xsl:template match="kml:outerBoundaryIs | kml:innerBoundaryIs">
        <xsl:element name="{local-name()}">
            <LinearRing>
                <xsl:apply-templates select="./kml:LinearRing/kml:coordinates"/>
            </LinearRing>   
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="kml:MultiGeometry">
        <MultiGeometry>
            <xsl:apply-templates select="kml:Polygon"/>  
        </MultiGeometry>
    </xsl:template>
    
    <xsl:template match="kml:Point">
        <Point>
            <xsl:apply-templates select="kml:coordinates"/>
        </Point>
    </xsl:template>

    <xsl:template match="kml:coordinates">
        <xsl:call-template name="parse-coordinates">
            <xsl:with-param name="str" select="normalize-space(.)" />
        </xsl:call-template>
    </xsl:template>
    
    <xsl:template name="parse-coordinates">
        <xsl:param name="str" />
        <xsl:choose>
            <xsl:when test="contains($str,' ')">      
                <xsl:variable name="coord"><xsl:value-of select="substring-before($str,' ')" /></xsl:variable>
                <xsl:variable name="longitude"><xsl:value-of select="substring-before($coord,',')" /></xsl:variable>
                <xsl:variable name="latitude"><xsl:value-of select="substring-before(substring-after($coord,','),',')" /></xsl:variable>
                <xsl:element name="coordinate">
                    <xsl:attribute name="longitude">
                        <xsl:value-of select="$longitude"/>
                    </xsl:attribute>
                    <xsl:attribute name="latitude">
                        <xsl:value-of select="$latitude" />
                    </xsl:attribute>
                 </xsl:element>
                <xsl:call-template name="parse-coordinates">
                    <xsl:with-param name="str" select="normalize-space(substring-after($str,' '))" />
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:if test="string-length($str) > 0">
                    <xsl:variable name="longitude"><xsl:value-of select="substring-before($str,',')" /></xsl:variable>
                    <xsl:variable name="latitude"><xsl:value-of select="substring-before(substring-after($str,','),',')" /></xsl:variable>
                    <xsl:element name="coordinate">
                        <xsl:attribute name="longitude">
                            <xsl:value-of select="$longitude"/>
                        </xsl:attribute>
                        <xsl:attribute name="latitude">
                            <xsl:value-of select="$latitude" />
                        </xsl:attribute>
                    </xsl:element>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>
